#
#  This software was derived from SATAN 1.1.1 by Dan Farmer and Wietse Venema
#  (http://www.porcupine.org/satan).
#
# miscellaneous perl functions that don't belong anywhere else
#

#
# Symbolic offsets for SARA record fields.
#

$TARGET_FIELD = 0;
$SERVICE_FIELD = 1;
$STATUS_FIELD = 2;
$SEVERITY_FIELD = 3;
$TRUSTEE_FIELD = 4;
$TRUSTED_FIELD = 5;
$SERVICE_OUTPUT_FIELD = 6;
$TEXT_FIELD = 7;
$SARA_FIELDS = 8;

# strip leading directory and optional suffix information.
sub basename {
    local($path, $suffix) = @_;

    $path =~ s:.*/::;
    if ($suffix) {
	$path =~ s/$suffix$//;
    }
    return $path;
}

# print to string 

sub sara_string{

    return "$target|$service|$status|$severity|$trustee|$trusted|$service_output|$text";
}

# print the output for the brain/optimizer
sub sara_print {

    print "$target|$service|$status|$severity|$trustee|$trusted|$service_output|$text\n";

}

# format the output for the brain/optimizer
sub sara_text {

    return "$target|$service|$status|$severity|$trustee|$trusted|$service_output|$text";

}

# breakup record into its constituent fields
sub sara_split {
    local ($line) = @_;

    return ((($target,$service,$status,$severity,$trustee,$trusted,$service_output,$text) = split(/\|/, $line)) != $SARA_FIELDS);
}

# count the number of elements in an associative array.
sub sizeof {
    local(*which) = @_;
    local(@keywords);
 
    @keywords = keys %which;
    return($#keywords + 1);
}

#
# ensure that all paths in paths.pl file actually exist and are executable
sub check_paths {
local($caps, $command, $path, $null);

die "Can't open paths.pl\n" unless open(PATHS, "paths.pl");

while (<PATHS>) {
	($caps, $command) = split(/=/, $_);
	($null, $path, $null) = split(/\"/, $command);
	die "$path is *not* executable - necessary for SARA to run\n"
		unless -x $path;
	}

close(PATHS);
}
sub delay_sara {
  local($delay) = 0;
  local($cnt)   = 0;
  $delay_file="rules/timing";
  if (open(DELAY,$delay_file)) {
     while(<DELAY>) {
      chomp;
      next if /#/ || /^$/;
      $delays[$cnt] = $_;
      local($delay_hours,$delay_days,$delay_dates) = split(/\|/,$delays[$cnt]);
      if($delay_hours!~/[0-2][0-9][0-9][0-9]-[[0-2][0-9][0-9][0-9]/) {
        if ($delay_hours !~ /\*/) {
         print "  Error in rules/timing: illegal entry in hours field\n";
         $delays[$cnt] = "";
       }
      }
      $_ = $delay_days;
      if(!/Sun/i && !/Mon/i && !/Tue/i && !/Wed/i && !/Thu/i) {
        if(!/Fri/ && !/Sat/i && !/\*/){
          print "  Error in rules/timing: illegal entry in days field\n";
          $delays[$cnt] = "";
        }
      }
      $_ = $delay_dates;
      if (!/Jan/i && !/Feb/i && !/Mar/i && !/Apr/i) {
       if (!/May/i && !/Jun/i && !/Jul/i && !/Aug/i) {
        if (!/Sep/i && !/Oct/i && !/Nov/i && !/Dec/i && !/\*/) {
         print "  Error in rules/timing: illegal entry in dates field\n";
         $delays[$cnt] = "";
        }
       }
      }
    $cnt++;
    }
    if (@delays) {
      $doit_delay = 1;
      print STDOUT "  Timing restrictions enforced by rules/timing\n";
      &do_delays();
      return 1;
    } else {
      return 0;
    }
   } else {
     return 0;
   }
}
sub do_delays() {
  $delay_flag = 1;
  $update_flag =0;
  while ($delay_flag == 1) {
   $delay_flag = 0;
   for (@delays) {
    local($delay_hours,$delay_days,$delay_dates) = split(/\|/);
    local($start_hr,$stop_hr) = split(/-/,$delay_hours);
     $tmp=delay_timer("hours");
     if(($tmp >= $start_hr && $tmp <= $stop_hr) || $delay_hours eq "\*") {
      if ($delay_days && $delay_hours && $delay_dates) {
        if($delay_days eq delay_timer("week") || $delay_days eq "\*") {
          if ($delay_dates eq delay_timer("date")|| $delay_dates eq "\*") {
            &update_status("SARA suspended- rules/timing") if $update_flag == 0;
            $update_flag = 1;
            sleep(60);
            $delay_flag = 1;
          }
        }
      }
     } 
   }
  }
}
sub delay_timer() {
  local($arg) = "@_";
  ($now_day_week,$now_month,$now_day,$now_hours)=split(/\s+/,`date`);
  ($tmp_hour, $tmp_min) = split(/\:/,$now_hours);
  $now_date = "$now_day $now_month";
  $now_hours = "$tmp_hour$tmp_min";
  return "$now_date" if $arg eq "date";
  return "$now_hours" if $arg eq "hours";
  return "$now_day_week" if $arg eq "week";
  return 0;
}

sub get_administrator {
#
# Interogate Administrator database for IP matches
# $_ip is the IP to search for
#
  $a = $b = $c = $network = @results = "";
  $admins ="";

  ($a,$b,$c) = split(/\./,$_ip);
  $network = "$a.$b.$c";
  @results = `grep $network administrators/*`;
  for (@results) {
    next if /^#/;
    ($admin, $ip) = split(/:/);
    $ip =~ s/\n//;
    $admin =~ s/administrators\///;
    if ($ip =~ /\-/) {
      ($ip_start,$ip_stop) = split(/\-/,$ip);
      ($dummy,$dummy,$dummy,$start) = split(/\./,$ip_start);
      ($dummy,$dummy,$dummy,$stop)  = split(/\./,$ip_stop);
      ($dummy,$dummy,$dummy,$mine) = split(/\./,$_ip);
      if (($mine >= $start) && ($mine <= $stop)) {
         $admins .= "$admin\;";
      }
    } elsif ($_ip eq $ip) {
      $admins .= "$admin\;";
    }
  }
  $admins =~ s/\;$//;
  return $admins;
}
